@@ -8,6 +8,19 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
+use std::{
+    borrow::Cow,
+    cell::RefCell,
+    error, fmt,
+    io::Write,
+    panic,
+    sync::atomic::{AtomicUsize, Ordering::SeqCst},
+};
+
+use scoped_tls::scoped_thread_local;
+#[cfg(feature = "tty-emitter")]
+use termcolor::{Color, ColorSpec};
+
 use self::Level::*;
 pub use self::{
     diagnostic::{Diagnostic, DiagnosticId, DiagnosticStyledString, SubDiagnostic},
@@ -20,17 +33,6 @@ use crate::{
     sync::{Lock, LockCell, Lrc},
     syntax_pos::{BytePos, FileLinesResult, FileName, Loc, MultiSpan, Span, NO_EXPANSION},
 };
-use scoped_tls::scoped_thread_local;
-use std::{
-    borrow::Cow,
-    cell::RefCell,
-    error, fmt,
-    io::Write,
-    panic,
-    sync::atomic::{AtomicUsize, Ordering::SeqCst},
-};
-#[cfg(feature = "tty-emitter")]
-use termcolor::{Color, ColorSpec};
 
 mod diagnostic;
 mod diagnostic_builder;
@@ -44,6 +46,10 @@ mod styled_buffer;
     feature = "diagnostic-serde",
     derive(serde::Serialize, serde::Deserialize)
 )]
+#[cfg_attr(
+    feature = "plugin-base",
+    derive(rkyv::Archive, rkyv::Serialize, rkyv::Deserialize)
+)]
 pub enum Applicability {
     MachineApplicable,
     HasPlaceholders,
@@ -56,6 +62,10 @@ pub enum Applicability {
     feature = "diagnostic-serde",
     derive(serde::Serialize, serde::Deserialize)
 )]
+#[cfg_attr(
+    feature = "plugin-base",
+    derive(rkyv::Archive, rkyv::Serialize, rkyv::Deserialize)
+)]
 pub struct CodeSuggestion {
     /// Each substitute can have multiple variants due to multiple
     /// applicable suggestions
@@ -103,6 +113,10 @@ pub struct CodeSuggestion {
     feature = "diagnostic-serde",
     derive(serde::Serialize, serde::Deserialize)
 )]
+#[cfg_attr(
+    feature = "plugin-base",
+    derive(rkyv::Archive, rkyv::Serialize, rkyv::Deserialize)
+)]
 pub struct Substitution {
     pub parts: Vec<SubstitutionPart>,
 }
@@ -112,6 +126,10 @@ pub struct Substitution {
     feature = "diagnostic-serde",
     derive(serde::Serialize, serde::Deserialize)
 )]
+#[cfg_attr(
+    feature = "plugin-base",
+    derive(rkyv::Archive, rkyv::Serialize, rkyv::Deserialize)
+)]
 pub struct SubstitutionPart {
     pub span: Span,
     pub snippet: String,
@@ -452,6 +470,7 @@ impl Handler {
         }
         result
     }
+
     pub fn struct_span_warn_with_code<'a, S: Into<MultiSpan>>(
         &'a self,
         sp: S,
@@ -466,6 +485,7 @@ impl Handler {
         }
         result
     }
+
     pub fn struct_warn<'a>(&'a self, msg: &str) -> DiagnosticBuilder<'a> {
         let mut result = DiagnosticBuilder::new(self, Level::Warning, msg);
         if !self.flags.can_emit_warnings {
@@ -473,6 +493,7 @@ impl Handler {
         }
         result
     }
+
     pub fn struct_span_err<'a, S: Into<MultiSpan>>(
         &'a self,
         sp: S,
@@ -482,6 +503,7 @@ impl Handler {
         result.set_span(sp);
         result
     }
+
     pub fn struct_span_err_with_code<'a, S: Into<MultiSpan>>(
         &'a self,
         sp: S,
@@ -493,11 +515,13 @@ impl Handler {
         result.code(code);
         result
     }
+
     // FIXME: This method should be removed (every error should have an associated
     // error code).
     pub fn struct_err<'a>(&'a self, msg: &str) -> DiagnosticBuilder<'a> {
         DiagnosticBuilder::new(self, Level::Error, msg)
     }
+
     pub fn struct_err_with_code<'a>(
         &'a self,
         msg: &str,
@@ -507,6 +531,7 @@ impl Handler {
         result.code(code);
         result
     }
+
     pub fn struct_span_fatal<'a, S: Into<MultiSpan>>(
         &'a self,
         sp: S,
@@ -516,6 +541,7 @@ impl Handler {
         result.set_span(sp);
         result
     }
+
     pub fn struct_span_fatal_with_code<'a, S: Into<MultiSpan>>(
         &'a self,
         sp: S,
@@ -527,6 +553,7 @@ impl Handler {
         result.code(code);
         result
     }
+
     pub fn struct_fatal<'a>(&'a self, msg: &str) -> DiagnosticBuilder<'a> {
         DiagnosticBuilder::new(self, Level::Fatal, msg)
     }
@@ -545,6 +572,7 @@ impl Handler {
         self.emit(&sp.into(), msg, Fatal);
         FatalError
     }
+
     pub fn span_fatal_with_code<S: Into<MultiSpan>>(
         &self,
         sp: S,
@@ -554,9 +582,11 @@ impl Handler {
         self.emit_with_code(&sp.into(), msg, code, Fatal);
         FatalError
     }
+
     pub fn span_err<S: Into<MultiSpan>>(&self, sp: S, msg: &str) {
         self.emit(&sp.into(), msg, Error);
     }
+
     pub fn mut_span_err<'a, S: Into<MultiSpan>>(
         &'a self,
         sp: S,
@@ -566,19 +596,24 @@ impl Handler {
         result.set_span(sp);
         result
     }
+
     pub fn span_err_with_code<S: Into<MultiSpan>>(&self, sp: S, msg: &str, code: DiagnosticId) {
         self.emit_with_code(&sp.into(), msg, code, Error);
     }
+
     pub fn span_warn<S: Into<MultiSpan>>(&self, sp: S, msg: &str) {
         self.emit(&sp.into(), msg, Warning);
     }
+
     pub fn span_warn_with_code<S: Into<MultiSpan>>(&self, sp: S, msg: &str, code: DiagnosticId) {
         self.emit_with_code(&sp.into(), msg, code, Warning);
     }
+
     pub fn span_bug<S: Into<MultiSpan>>(&self, sp: S, msg: &str) -> ! {
         self.emit(&sp.into(), msg, Bug);
         panic!("{}", ExplicitBug);
     }
+
     pub fn delay_span_bug<S: Into<MultiSpan>>(&self, sp: S, msg: &str) {
         if self.flags.treat_err_as_bug {
             // FIXME: don't abort here if report_delayed_bugs is off
@@ -588,29 +623,36 @@ impl Handler {
         diagnostic.set_span(sp.into());
         self.delay_as_bug(diagnostic);
     }
+
     fn delay_as_bug(&self, diagnostic: Diagnostic) {
         if self.flags.report_delayed_bugs {
             DiagnosticBuilder::new_diagnostic(self, diagnostic.clone()).emit();
         }
         self.delayed_span_bugs.borrow_mut().push(diagnostic);
     }
+
     pub fn span_bug_no_panic<S: Into<MultiSpan>>(&self, sp: S, msg: &str) {
         self.emit(&sp.into(), msg, Bug);
     }
+
     pub fn span_note_without_error<S: Into<MultiSpan>>(&self, sp: S, msg: &str) {
         self.emit(&sp.into(), msg, Note);
     }
+
     pub fn span_note_diag<'a>(&'a self, sp: Span, msg: &str) -> DiagnosticBuilder<'a> {
         let mut db = DiagnosticBuilder::new(self, Note, msg);
         db.set_span(sp);
         db
     }
+
     pub fn span_unimpl<S: Into<MultiSpan>>(&self, sp: S, msg: &str) -> ! {
         self.span_bug(sp, &format!("unimplemented {}", msg));
     }
+
     pub fn failure(&self, msg: &str) {
         DiagnosticBuilder::new(self, FailureNote, msg).emit()
     }
+
     pub fn fatal(&self, msg: &str) -> FatalError {
         if self.flags.treat_err_as_bug {
             self.bug(msg);
@@ -618,6 +660,7 @@ impl Handler {
         DiagnosticBuilder::new(self, Fatal, msg).emit();
         FatalError
     }
+
     pub fn err(&self, msg: &str) {
         if self.flags.treat_err_as_bug {
             self.bug(msg);
@@ -625,19 +668,23 @@ impl Handler {
         let mut db = DiagnosticBuilder::new(self, Error, msg);
         db.emit();
     }
+
     pub fn warn(&self, msg: &str) {
         let mut db = DiagnosticBuilder::new(self, Warning, msg);
         db.emit();
     }
+
     pub fn note_without_error(&self, msg: &str) {
         let mut db = DiagnosticBuilder::new(self, Note, msg);
         db.emit();
     }
+
     pub fn bug(&self, msg: &str) -> ! {
         let mut db = DiagnosticBuilder::new(self, Bug, msg);
         db.emit();
         panic!("{}", ExplicitBug);
     }
+
     pub fn unimpl(&self, msg: &str) -> ! {
         self.bug(&format!("unimplemented {}", msg));
     }
@@ -709,6 +756,7 @@ impl Handler {
         }
         FatalError.raise();
     }
+
     pub fn emit(&self, msp: &MultiSpan, msg: &str, lvl: Level) {
         if lvl == Warning && !self.flags.can_emit_warnings {
             return;
@@ -720,6 +768,7 @@ impl Handler {
             self.abort_if_errors();
         }
     }
+
     pub fn emit_with_code(&self, msp: &MultiSpan, msg: &str, code: DiagnosticId, lvl: Level) {
         if lvl == Warning && !self.flags.can_emit_warnings {
             return;
@@ -786,6 +835,10 @@ impl Handler {
     feature = "diagnostic-serde",
     derive(serde::Serialize, serde::Deserialize)
 )]
+#[cfg_attr(
+    feature = "plugin-base",
+    derive(rkyv::Archive, rkyv::Serialize, rkyv::Deserialize)
+)]
 pub enum Level {
     Bug,
     Fatal,
