@@ -16,16 +16,6 @@
 //! `spans` and used pervasively in the compiler. They are absolute positions
 //! within the SourceMap, which upon request can be converted to line and column
 //! information, source code snippets, etc.
-pub use crate::syntax_pos::*;
-use crate::{
-    collections::AHashMap,
-    errors::SourceMapper,
-    rustc_data_structures::stable_hasher::StableHasher,
-    sync::{Lock, LockGuard, Lrc, MappedLockGuard},
-};
-use once_cell::sync::Lazy;
-#[cfg(feature = "sourcemap")]
-use sourcemap::SourceMapBuilder;
 use std::{
     cmp,
     cmp::{max, min},
@@ -35,8 +25,20 @@ use std::{
     path::{Path, PathBuf},
     sync::atomic::{AtomicUsize, Ordering::SeqCst},
 };
+
+use once_cell::sync::Lazy;
+#[cfg(feature = "sourcemap")]
+use sourcemap::SourceMapBuilder;
 use tracing::debug;
 
+pub use crate::syntax_pos::*;
+use crate::{
+    collections::AHashMap,
+    errors::SourceMapper,
+    rustc_data_structures::stable_hasher::StableHasher,
+    sync::{Lock, LockGuard, Lrc, MappedLockGuard},
+};
+
 static CURRENT_DIR: Lazy<Option<PathBuf>> = Lazy::new(|| env::current_dir().ok());
 
 // _____________________________________________________________________________
@@ -579,6 +581,16 @@ impl SourceMap {
         })
     }
 
+    /// Calls `op` with the source code located at `sp`.
+    pub fn with_snippet_of_span<F, Ret>(&self, sp: Span, op: F) -> Result<Ret, SpanSnippetError>
+    where
+        F: FnOnce(&str) -> Ret,
+    {
+        self.span_to_source(sp, |src, start_index, end_index| {
+            op(&src[start_index..end_index])
+        })
+    }
+
     pub fn span_to_margin(&self, sp: Span) -> Option<usize> {
         match self.span_to_prev_source(sp) {
             Err(_) => None,
@@ -1190,21 +1202,27 @@ impl SourceMapper for SourceMap {
     fn lookup_char_pos(&self, pos: BytePos) -> Loc {
         self.lookup_char_pos(pos)
     }
+
     fn span_to_lines(&self, sp: Span) -> FileLinesResult {
         self.span_to_lines(sp)
     }
+
     fn span_to_string(&self, sp: Span) -> String {
         self.span_to_string(sp)
     }
+
     fn span_to_filename(&self, sp: Span) -> FileName {
         self.span_to_filename(sp)
     }
+
     fn merge_spans(&self, sp_lhs: Span, sp_rhs: Span) -> Option<Span> {
         self.merge_spans(sp_lhs, sp_rhs)
     }
+
     fn call_span_if_macro(&self, sp: Span) -> Span {
         sp
     }
+
     fn doctest_offset_line(&self, line: usize) -> usize {
         self.doctest_offset_line(line)
     }
